import uuid
from typing import Any

from async_yookassa.apiclient import APIClient
from async_yookassa.enums.request_methods import HTTPMethodEnum
from async_yookassa.models.payment_capture import CapturePaymentRequest
from async_yookassa.models.payment_request import PaymentRequest
from async_yookassa.models.payment_response import PaymentListResponse, PaymentResponse
from async_yookassa.utils import get_base_headers


class Payment:
    """
    Класс, представляющий модель Payment.
    """

    base_path = "/payments"

    CMS_NAME = "async_yookassa_python"

    def __init__(self):
        self.client = APIClient()

    @classmethod
    async def find_one(cls, payment_id: str) -> PaymentResponse:
        """
        Возвращает информацию о платеже

        :param payment_id: Уникальный идентификатор платежа
        :return: Объект ответа PaymentResponse, возвращаемого API при запросе платежа
        """
        instance = cls()
        if not isinstance(payment_id, str):
            raise ValueError("Invalid payment_id value")

        path = instance.base_path + "/" + payment_id

        response = await instance.client.request(method=HTTPMethodEnum.GET, path=path)
        return PaymentResponse(**response)

    @classmethod
    async def create(
        cls, params: dict[str, Any] | PaymentRequest, idempotency_key: uuid.UUID | None = None
    ) -> PaymentResponse:
        """
        Создание платежа

        :param params: Данные передаваемые в API
        :param idempotency_key: Ключ идемпотентности
        :return: Объект ответа PaymentResponse, возвращаемого API при запросе платежа
        """
        instance = cls()

        path = instance.base_path

        headers = get_base_headers(idempotency_key=idempotency_key)

        if isinstance(params, dict):
            params_object = PaymentRequest(**params)
        elif isinstance(params, PaymentRequest):
            params_object = params
        else:
            raise TypeError("Invalid params value type")

        params_object = instance.add_default_cms_name(params_object=params_object)

        response = await instance.client.request(
            body=params_object, method=HTTPMethodEnum.POST, path=path, query_params=None, headers=headers
        )
        return PaymentResponse(**response)

    @classmethod
    async def capture(
        cls,
        payment_id: str,
        params: dict[str, Any] | CapturePaymentRequest | None = None,
        idempotency_key: uuid.UUID | None = None,
    ) -> PaymentResponse:
        """
        Подтверждение платежа

        :param payment_id: Уникальный идентификатор платежа
        :param params: Данные передаваемые в API
        :param idempotency_key: Ключ идемпотентности
        :return: Объект ответа PaymentResponse, возвращаемого API при запросе платежа
        """
        instance = cls()
        if not isinstance(payment_id, str):
            raise ValueError("Invalid payment_id value")

        path = instance.base_path + "/" + payment_id + "/capture"

        headers = get_base_headers(idempotency_key=idempotency_key)

        if isinstance(params, dict):
            params_object = CapturePaymentRequest(**params)
        elif isinstance(params, CapturePaymentRequest):
            params_object = params
        else:
            params_object = None

        response = await instance.client.request(
            body=params_object, method=HTTPMethodEnum.POST, path=path, headers=headers
        )
        return PaymentResponse(**response)

    @classmethod
    async def cancel(cls, payment_id: str, idempotency_key: uuid.UUID | None = None) -> PaymentResponse:
        """
        Отмена платежа

        :param payment_id: Уникальный идентификатор платежа
        :param idempotency_key: Ключ идемпотентности
        :return: Объект ответа PaymentResponse, возвращаемого API при запросе платежа
        """
        instance = cls()
        if not isinstance(payment_id, str):
            raise ValueError("Invalid payment_id value")

        path = instance.base_path + "/" + payment_id + "/cancel"

        headers = get_base_headers(idempotency_key=idempotency_key)

        response = await instance.client.request(method=HTTPMethodEnum.POST, path=path, headers=headers)
        return PaymentResponse(**response)

    @classmethod
    async def list(cls, params: dict[str, Any] | None = None) -> PaymentListResponse:
        """
        Возвращает список платежей

        :param params: Данные передаваемые в API
        :return: Объект ответа PaymentListResponse, возвращаемого API при запросе списка платежей
        """
        instance = cls()

        path = cls.base_path

        response = await instance.client.request(method=HTTPMethodEnum.GET, path=path, query_params=params)
        return PaymentListResponse(**response)

    def add_default_cms_name(self, params_object: PaymentRequest) -> PaymentRequest:
        """
        Добавляет cms_name в metadata со значением по умолчанию

        :param params_object: Данные передаваемые в API
        :return: PaymentRequest Объект запроса к API
        """
        if not params_object.metadata:
            params_object.metadata = {"cms_name": self.CMS_NAME}

        if "cms_name" not in params_object.metadata:
            params_object.metadata["cms_name"] = self.CMS_NAME

        return params_object



from enum import Enum


class CardTypeEnum(str, Enum):
    MasterCard = "MasterCard"
    Visa = "Visa"
    Mir = "Mir"
    UnionPay = "UnionPay"
    JCB = "JCB"
    AmericanExpress = "AmericanExpress"
    DinersClub = "DinersClub"
    DiscoverCard = "DiscoverCard"
    InstaPayment = "InstaPayment"
    InstaPaymentTM = "InstaPaymentTM"
    Laser = "Laser"
    Dankort = "Dankort"
    Solo = "Solo"
    Switch = "Switch"
    Unknown = "Unknown"


class SourceEnum(str, Enum):
    mir_pay = "mir_pay"
    apple_pay = "apple_pay"
    google_pay = "google_pay"



from async_yookassa.exceptions.api_error import APIError


class BadRequestError(APIError):
    """
    Неправильный запрос. Чаще всего этот статус выдается из-за нарушения правил взаимодействия с API.
    """

    HTTP_CODE = 400



from pydantic import BaseModel, Field

from async_yookassa.enums.receipt_type import ReceiptType
from async_yookassa.models.payment_submodels.deal_submodels.settlements import (
    SettlementReceipt,
)
from async_yookassa.models.payment_submodels.receipt_submodels.customer import Customer
from async_yookassa.models.payment_submodels.receipt_submodels.payment_subject_industry_details import (
    PaymentSubjectIndustryDetails,
)
from async_yookassa.models.payment_submodels.receipt_submodels.receipt_item import (
    ReceiptItem,
)
from async_yookassa.models.payment_submodels.receipt_submodels.receipt_operational_details import (
    ReceiptOperationalDetails,
)
from async_yookassa.models.receipt_submodels.additional_user_props import (
    AdditionalUserProps,
)


class ReceiptRequest(BaseModel):
    type: ReceiptType
    payment_id: str | None = None
    refund_id: str | None = None
    customer: Customer
    items: list[ReceiptItem]
    send: bool = True
    tax_system_code: int | None = Field(ge=1, le=6, default=None)
    additional_user_props: AdditionalUserProps | None = None
    receipt_industry_details: list[PaymentSubjectIndustryDetails] | None = None
    receipt_operational_details: ReceiptOperationalDetails | None = None
    settlements: list[SettlementReceipt] | None = None
    on_behalf_of: str | None = None


class ReceiptListResponse(BaseModel):
    type: str
    items: list[ReceiptRequest]
    next_cursor: str | None = None



from typing import Any

from pydantic import BaseModel, ConfigDict, Field, model_validator

from async_yookassa.enums.payment_method import PaymentMethodTypeEnum
from async_yookassa.models.payment_submodels.amount import Amount
from async_yookassa.models.payment_submodels.payment_method_submodels.articles import (
    ArticleRefund,
    ArticleResponse,
)
from async_yookassa.models.payment_submodels.payment_method_submodels.card import (
    CardResponse,
)
from async_yookassa.models.payment_submodels.payment_method_submodels.electronic_certificate import (
    ElectronicCertificate,
)
from async_yookassa.models.payment_submodels.payment_method_submodels.payer_bank_details import (
    B2BSBPayerBankDetails,
    SBPPayerBankDetails,
)
from async_yookassa.models.payment_submodels.payment_method_submodels.vat_data import (
    VatData,
)


class PaymentMethodBase(BaseModel):
    type: PaymentMethodTypeEnum
    electronic_certificate: ElectronicCertificate | None = None
    sbp_operation_id: str | None = None

    model_config = ConfigDict(use_enum_values=True)


class PaymentMethod(PaymentMethodBase):
    id: str
    saved: bool
    title: str | None = None
    discount_amount: Amount | None = None
    loan_option: str | None = None
    login: str | None = None
    payer_bank_details: SBPPayerBankDetails | B2BSBPayerBankDetails | None = None
    payment_purpose: str | None = Field(max_length=210, default=None)
    vat_data: VatData | None = None
    articles: ArticleResponse | None = None
    card: CardResponse | None = None
    account_number: str | None = None
    phone: str | None = None

    @model_validator(mode="before")
    def validate_required_fields(cls, values: dict[str, Any]):
        type_value = values.get("type")

        if type_value == PaymentMethodTypeEnum.sbp:
            if data := values.get("payer_bank_details"):
                values["payer_bank_details"] = SBPPayerBankDetails(**data)
        elif type_value == PaymentMethodTypeEnum.b2b_sberbank:
            if not values.get("payment_purpose"):
                raise ValueError("Field 'payment_purpose' are required for type 'b2b_sberbank'")
            if not values.get("vat_data"):
                raise ValueError("Field 'vat_data' are required for type 'b2b_sberbank'")
            if data := values.get("payer_bank_details"):
                values["payer_bank_details"] = B2BSBPayerBankDetails(**data)

        return values


class PaymentMethodRefund(PaymentMethodBase):
    articles: ArticleRefund | None = None



from pydantic import BaseModel, Field

from async_yookassa.models.payment_submodels.amount import Amount


class Cart(BaseModel):
    description: str = Field(min_length=1, max_length=128)
    price: Amount
    discount_price: Amount | None = None
    quantity: int | float



from datetime import datetime
from typing import Any

from pydantic import BaseModel, Field

from async_yookassa.models.invoice_submodels.cart import Cart
from async_yookassa.models.invoice_submodels.delivery_method import DeliveryMethod
from async_yookassa.models.payment_request import PaymentData


class InvoiceRequest(BaseModel):
    payment_data: PaymentData
    cart: Cart
    delivery_method_data: DeliveryMethod
    expires_at: datetime
    locale: str | None = None
    description: str | None = Field(max_length=128, default=None)
    metadata: dict[str, Any] | None = None



from pydantic import BaseModel

from async_yookassa.models.payment_submodels.authorization_details_submodels.three_d_secure import (
    ThreeDSecure,
)


class AuthorizationDetails(BaseModel):
    rrn: str | None = None
    auth_code: str | None = None
    three_d_secure: ThreeDSecure



from enum import Enum


class AgentTypeEnum(str, Enum):
    banking_payment_agent = "banking_payment_agent"
    banking_payment_subagent = "banking_payment_subagent"
    payment_agent = "payment_agent"
    payment_subagent = "payment_subagent"
    attorney = "attorney"
    commissioner = "commissioner"
    agent = "agent"



from pydantic import BaseModel, Field

from async_yookassa.enums.payment_details import PaymentDetailsStatusEnum


class PaymentDetails(BaseModel):
    id: str = Field(min_length=39, max_length=39)
    status = PaymentDetailsStatusEnum



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field

from async_yookassa.enums.payment_response import ReceiptRegistrationEnum
from async_yookassa.enums.receipt_type import ReceiptType
from async_yookassa.models.payment_submodels.deal_submodels.settlements import (
    SettlementReceipt,
)
from async_yookassa.models.payment_submodels.receipt_submodels.payment_subject_industry_details import (
    PaymentSubjectIndustryDetails,
)
from async_yookassa.models.payment_submodels.receipt_submodels.receipt_item import (
    ReceiptItem,
)
from async_yookassa.models.payment_submodels.receipt_submodels.receipt_operational_details import (
    ReceiptOperationalDetails,
)


class ReceiptResponse(BaseModel):
    id: str = Field(min_length=39, max_length=39)
    type: ReceiptType
    payment_id: str | None = None
    refund_id: str | None = None
    status: ReceiptRegistrationEnum
    fiscal_document_number: str | None = None
    fiscal_storage_number: str | None = None
    fiscal_attribute: str | None = None
    registered_at: datetime | None = None
    fiscal_provider_id: str | None = None
    items: list[ReceiptItem]
    settlements: list[SettlementReceipt] | None = None
    on_behalf_of: str | None = None
    tax_system_code: int | None = None
    receipt_industry_details: list[PaymentSubjectIndustryDetails] | None = None
    receipt_operational_details: ReceiptOperationalDetails | None = None

    model_config = ConfigDict(use_enum_values=True)



from pydantic import BaseModel, ConfigDict

from async_yookassa.enums.invoice_cancellation_details import (
    InvoicePartyEnum,
    InvoiceReasonEnum,
)


class InvoiceCancellationDetails(BaseModel):
    party: InvoicePartyEnum
    reason: InvoiceReasonEnum

    model_config = ConfigDict(use_enum_values=True)



from typing import Any

from pydantic import BaseModel, Field

from async_yookassa.models.payment_submodels.amount import Amount
from async_yookassa.models.payment_submodels.payment_method_submodels.electronic_certificate import (
    Certificate,
)


class ArticleBase(BaseModel):
    article_number: int = Field(ge=1, le=999)
    tru_code: str = Field(min_length=30, max_length=30)
    article_code: str | None = Field(max_length=128, default=None)


class Article(ArticleBase):
    article_name: str = Field(max_length=128, default=None)
    quantity: int
    price: Amount
    metadata: dict[str, Any]


class ArticleResponse(ArticleBase):
    certificates: list[Certificate]


class ArticleRefund(ArticleBase):
    payment_article_number: int = Field(ge=1, le=999)
    quantity: int



from pydantic import BaseModel, ConfigDict

from async_yookassa.enums.cancellation_details import (
    PartyEnum,
    PartyEnumBase,
    ReasonEnum,
    ReasonRefundEnum,
)


class CancellationDetails(BaseModel):
    party: PartyEnum
    reason: ReasonEnum

    model_config = ConfigDict(use_enum_values=True)


class RefundDetails(BaseModel):
    party: PartyEnumBase
    reason: ReasonRefundEnum

    model_config = ConfigDict(use_enum_values=True)



from async_yookassa.exceptions.api_error import APIError


class ForbiddenError(APIError):
    """
    Секретный ключ или OAuth-токен верный, но не хватает прав для совершения операции.
    """

    HTTP_CODE = 403
